{
  "type": "record",
  "name": "DataProcessInfo",
  "namespace": "com.linkedin.pegasus2avro.dataprocess",
  "doc": "The inputs and outputs of this data process",
  "fields": [
    {
      "name": "inputs",
      "type": [
        "null",
        {
          "type": "array",
          "items": "string"
        }
      ],
      "doc": "the inputs of the data process",
      "default": null,
      "Relationship": {
        "/*": {
          "entityTypes": [
            "dataset"
          ],
          "isLineage": true,
          "name": "Consumes"
        }
      },
      "Searchable": {
        "/*": {
          "fieldName": "inputs",
          "fieldType": "URN",
          "numValuesFieldName": "numInputDatasets",
          "queryByDefault": false
        }
      }
    },
    {
      "name": "outputs",
      "type": [
        "null",
        {
          "type": "array",
          "items": "string"
        }
      ],
      "doc": "the outputs of the data process",
      "default": null,
      "Relationship": {
        "/*": {
          "entityTypes": [
            "dataset"
          ],
          "isLineage": true,
          "name": "Consumes"
        }
      },
      "Searchable": {
        "/*": {
          "fieldName": "outputs",
          "fieldType": "URN",
          "numValuesFieldName": "numOutputDatasets",
          "queryByDefault": false
        }
      }
    }
  ],
  "Aspect": {
    "name": "dataProcessInfo"
  }
}